package dev.shadowsoffire.apotheosis.adventure.client;

import net.minecraft.client.gui.narration.NarrationElementOutput;
import net.minecraft.util.Mth;
import net.minecraft.client.gui.GuiGraphics;
import net.minecraft.network.chat.Component;
import java.util.List;
import net.minecraft.client.gui.components.AbstractWidget;

public abstract class DropDownList<T> extends AbstractWidget
{
    public static final int NO_SELECTION = -1;
    protected final int maxDisplayedEntries;
    protected final int baseHeight;
    protected List<T> entries;
    protected float scrollOffs;
    protected boolean scrolling;
    protected int startIndex;
    protected boolean isOpen;
    protected int selected;
    
    public DropDownList(final int x, final int y, final int width, final int height, final Component narrationMsg, final List<T> entries, final int maxDisplayedEntries) {
        super(x, y, width, height, narrationMsg);
        this.isOpen = false;
        this.selected = -1;
        this.entries = entries;
        this.maxDisplayedEntries = maxDisplayedEntries;
        this.baseHeight = height;
    }
    
    protected void m_87963_(final GuiGraphics gfx, final int mouseX, final int mouseY, final float partialTick) {
        final int selected = this.getSelected();
        if (selected != -1) {
            this.renderEntry(gfx, this.m_252754_(), this.m_252907_(), mouseX, mouseY, this.entries.get(selected));
        }
        if (this.isOpen) {
            for (int i = this.startIndex; i < this.startIndex + this.maxDisplayedEntries && i < this.entries.size(); ++i) {
                this.renderEntry(gfx, this.m_252754_(), this.m_252907_() + this.baseHeight * (1 + i - this.startIndex), mouseX, mouseY, this.entries.get(i));
            }
        }
    }
    
    public void m_5716_(final double mouseX, final double mouseY) {
        if (!this.isOpen) {
            if (this.entries.isEmpty()) {
                return;
            }
            this.isOpen = true;
            this.f_93619_ = this.baseHeight * (1 + Math.min(this.entries.size(), this.maxDisplayedEntries));
        }
        else {
            this.selected = this.getHoveredSlot(mouseX, mouseY);
            this.f_93619_ = this.baseHeight;
            this.isOpen = false;
        }
    }
    
    public static boolean isHovering(final int x, final int y, final int width, final int height, final double mouseX, final double mouseY) {
        return mouseX >= x && mouseY >= y && mouseX < x + width && mouseY < y + height;
    }
    
    public int getHoveredSlot(final double mouseX, final double mouseY) {
        if (isHovering(this.m_252754_(), this.m_252907_(), this.f_93618_, this.baseHeight, mouseX, mouseY)) {
            return this.getSelected();
        }
        if (this.isOpen) {
            for (int i = 0; i < this.maxDisplayedEntries; ++i) {
                if (this.startIndex + i < this.entries.size() && isHovering(this.m_252754_(), this.m_252907_() + (i + 1) * this.baseHeight, this.f_93618_, this.baseHeight, mouseX, mouseY)) {
                    return this.startIndex + i;
                }
            }
        }
        return -1;
    }
    
    public boolean m_7979_(final double pMouseX, final double pMouseY, final int pButton, final double pDragX, final double pDragY) {
        if (this.scrolling && this.isScrollBarActive()) {
            final int barTop = this.m_252754_() + 14;
            final int barBot = barTop + 103;
            this.scrollOffs = ((float)pMouseY - barTop - 6.0f) / (barBot - barTop - 12.0f) - 0.12f;
            this.scrollOffs = Mth.m_14036_(this.scrollOffs, 0.0f, 1.0f);
            this.startIndex = (int)(this.scrollOffs * this.getOffscreenRows() + 0.5);
            return true;
        }
        return super.m_7979_(pMouseX, pMouseY, pButton, pDragX, pDragY);
    }
    
    public boolean m_6050_(final double pMouseX, final double pMouseY, final double pDelta) {
        if (this.isScrollBarActive()) {
            final int i = this.getOffscreenRows();
            this.scrollOffs -= (float)(pDelta / i);
            this.scrollOffs = Mth.m_14036_(this.scrollOffs, 0.0f, 1.0f);
            this.startIndex = (int)(this.scrollOffs * i + 0.5);
        }
        return true;
    }
    
    private boolean isScrollBarActive() {
        return this.isOpen() && this.entries.size() > this.maxDisplayedEntries;
    }
    
    protected int getOffscreenRows() {
        return this.entries.size() - this.maxDisplayedEntries;
    }
    
    public int getSelected() {
        return Mth.m_14045_(this.selected, -1, this.entries.size() - 1);
    }
    
    public void setSelected(final int selected) {
        this.selected = Mth.m_14045_(selected, -1, this.entries.size() - 1);
    }
    
    public boolean isOpen() {
        return this.isOpen;
    }
    
    protected abstract void renderEntry(final GuiGraphics p0, final int p1, final int p2, final int p3, final int p4, final T p5);
    
    protected void m_168797_(final NarrationElementOutput output) {
    }
    
    public void setEntries(final List<T> entries) {
        this.entries = entries;
        this.selected = (this.entries.isEmpty() ? -1 : 0);
        this.f_93619_ = this.baseHeight;
        this.startIndex = 0;
        this.isOpen = false;
    }
}
